# Generated by Django 2.0.4 on 2018-04-17 16:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('speedinfo', '0004_auto_20180409_1416'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='viewprofiler',
            options={'verbose_name_plural': 'Views'},
        ),
    ]
